#ifndef SET_IMPLEMENTATION
#define SET_IMPLEMENTATION

/*****************************************************************************\
*                                                                             *
*  Name   : set                                                               *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1996-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "array.cpp"
#include "guards.h"
#include "set.h"
#include "utility.h"

namespace basis {

template <class contents>
bool set<contents>::member(const contents &to_test) const
{
  for (int i = 0; i < elements(); i++)
    if (to_test == this->get(i))
      return true;
  return false;
}

template <class contents>
bool set<contents>::add(const contents &to_add)
{
  if (this->member(to_add)) return false; 
  this->concatenate(to_add);
  return true;
}

template <class contents>
int set<contents>::find(const contents &to_find) const
{
  for (int i = 0; i < elements(); i++)
    if (to_find == this->get(i))
      return i;
  return common::NOT_FOUND;
}

template <class contents>
bool set<contents>::remove(const contents &to_remove)
{
  for (int i = 0; i < elements(); i++)
    if (to_remove == this->get(i)) {
      this->zap(i, i);
      return true;
    }
  return false;
}

template <class contents>
set<contents> set<contents>::intersection(const set &intersect_with) const
{
  set<contents> created(0, NIL, this->flags());
  const set *smaller = this;
  const set *larger = &intersect_with;
  if (this->elements() > intersect_with.elements()) {
    // switch the smaller one into place.
    smaller = &intersect_with;
    larger = this;
  }
  for (int i = 0; i < smaller->length(); i++)
    if (larger->member(smaller->get(i)))
      created.concatenate(smaller->get(i));
  return created;
}

template <class contents>
set<contents> set<contents>::set_union(const set &union_with) const
{
  set<contents> created = *this;
  for (int i = 0; i < union_with.elements(); i++)
    created.add(union_with.get(i));
  return created;
}

template <class contents>
void set<contents>::unionize(const set &union_with)
{
  for (int i = 0; i < union_with.elements(); i++)
    this->add(union_with.get(i));
}

template <class contents>
set<contents> set<contents>::difference(const set &differ_with) const
{
  set<contents> created = *this;
  for (int i = 0; i < differ_with.elements(); i++) {
    if (created.member(differ_with[i]))
      created.remove(differ_with[i]);
  }
  return created;
}

template <class contents>
void set<contents>::differentiate(const set &differ_with)
{
  for (int i = 0; i < differ_with.elements(); i++) {
    if (this->member(differ_with[i]))
      this->remove(differ_with[i]);
  }
}

}  // namespace.

#endif

