/*****************************************************************************\
*                                                                             *
*  Name   : rotating_byte_hasher                                              *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2001-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "byte_hasher.h"
#include "static_memory_gremlin.h"

#include <basis/utility.h>

hashing_algorithm *rotating_byte_hasher::clone() const
{ return new rotating_byte_hasher; }

u_int rotating_byte_hasher::hash(const void *key_data, int key_length) const
{
  if (!key_data) return 0;  // error!
  if (!key_length) return 0;  // ditto!

  byte *our_key = (byte *)key_data;
  byte hashed[4] = { 0, 0, 0, 0 };

  int fill_posn = 0;
  for (int i = 0; i < key_length; i++) {
    // add to the primary area.
    hashed[fill_posn] = hashed[fill_posn] + our_key[i];
    fill_posn++;
    if (fill_posn >= 4) fill_posn = 0;
    // add to the secondary area (the next in rotation after primary).
    hashed[fill_posn] = hashed[fill_posn] + (our_key[i] / 4);
  }
  
  u_int to_return = 0;
  for (int j = 0; j < 4; j++) to_return = (to_return << 8) + hashed[j];
  return to_return;
}

