#ifndef CONFIGURABLE_CLASS
#define CONFIGURABLE_CLASS

#include <basis/object_base.h>

// forward.
class configuration_list;

class configurable : public virtual object_base
{
public:
  IMPLEMENT_CLASS_NAME("configurable");
  virtual ~configurable() {}

  virtual void get_config(configuration_list &to_fill, bool append) const = 0;
    //!< interprets the contents of this object as a configuration list.
    /*!< the list of configlets can be stored in any configurator object.
    if the "append" flag is true, then the list is added to.  otherwise it
    is cleared first.  this method can also be used to retrieve the configlets
    that this class defines as its configuration interface.  that list can
    then be filled in using a configurator and passed to set_config(). */

  virtual bool set_config(const configuration_list &to_use) = 0;
    //!< retrieves the config items from "to_use" and stores them here.
    /*!< false is returned if any of the key items are missing or if the
    new key cannot be decoded. */
};

#endif

