#ifndef INT_HASH_CLASS
#define INT_HASH_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : int_hash                                                          *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2001-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "hash_table.h"

//! A hash table for storing integers.
/*!
  Implements a hash table indexed on integers that maintains a separate set of
  identifiers for listing the items that are presently in the hash table.  This
  slows down additions somewhat, but finds are not affected.  The advantage of
  the separate index is that the apply() method is much faster.
*/

template <class contents>
class int_hash : public hash_table<int, contents>
{
public:
  int_hash(int max_bits);
  ~int_hash();

  const int_set &ids() const;
  void ids(int_set &ids) const;
    //!< provides the current list of valid identifiers.

  outcome add(int key, contents *to_store);
    //!< overrides base add() and ensures that the id list stays up to date.
  contents *acquire(int key);
    //!< overrides base acquire() by ensuring that the ids stay up to date.
  bool zap(int key);
    //!< overrides base zap() method plus keeps id list updated.
  void reset();
    //!< overrides base reset() and ensures that the id list stays up to date.

  typedef bool apply_function(const int &key, contents &current,
        void *data_link);

  void apply(apply_function *to_apply, void *data_link);
    //!< operates on every item in the int_hash table.

private:
  int_set *_ids;
    //!< a separate list of the identifiers stored here.
    /*! this provides a fairly quick way to iterate rather than having to span
    the whole hash table.  it does slow down zap() a bit though. */
};

#endif

