#ifndef MANAGED_OBJECT_IMPLEMENTATION
#define MANAGED_OBJECT_IMPLEMENTATION

/*****************************************************************************\
*                                                                             *
*  Name   : managed_object                                                    *
*  Author : Aaron Buchanan                                                    *
*                                                                             *
*******************************************************************************
* Copyright (c) 2001-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "managed_object.h"

template <class contents>
class object_storer
{
public:
  object_storer(contents *_object)
  : m_Object(_object), m_refCount(1) {}
     // Constructor stores the original object and intializes the
     //  reference count to one.

  // When the storer is deleted we delete the original object
  ~object_storer()
  {
    delete m_Object;
    m_Object = NULL;
  }

  contents *GetObject() const   { return m_Object; }
    // Returns the originally stored object

  // Reference count for the number of references to the object
  void addRef()   { m_refCount++; }
  void removeRef()
  {
    // When the reference count reaches zero we delete the object_storer,
    // this in turn deletes the original object.
    m_refCount--;
    if( m_refCount <= 0 )
      delete this;
  }

private:
  contents *m_Object;
  int m_refCount;
};

////////////////////////////////////////////////////////////////////////////

template <class contents>
managed_object<contents>::managed_object()
: storer(NULL)
{}

template <class contents>
managed_object<contents>::managed_object(contents *&_object) 
: storer(new object_storer<contents>(_object))
{ _object = NULL; }

template <class contents>
managed_object<contents>::managed_object(const managed_object<contents> &_rhs)
: storer(NULL)
{ *this = _rhs; }

template <class contents>
managed_object<contents> &managed_object<contents>::operator =(const managed_object<contents> &rhs)
{
  if (this == &rhs) return *this;
  if(NULL != storer) storer->removeRef();
  storer = rhs.storer;
  if(NULL != storer) storer->addRef();
  return *this;
}

template <class contents>
managed_object<contents>::~managed_object()
{ if(NULL != storer) storer->removeRef(); }

template <class contents>
bool managed_object<contents>::operator ==(const managed_object<contents> &_compare) const
{
    if( (NULL == storer) || (NULL == _compare.storer) )
    {
        if( (NULL == storer) && (NULL == _compare.storer) )
            return true;
        return false;
    }
    return( *(storer->GetObject()) == *(_compare.storer->GetObject()) );
}

template <class contents>
managed_object<contents> *managed_object<contents>::new_reference()
{ return new managed_object<contents>(*this); }

template <class contents>
contents *managed_object<contents>::access()
{ return( (NULL == storer) ? NULL : storer->GetObject() ); }

template <class contents>
const contents *managed_object<contents>::observe() const
{ return( (NULL == storer) ? NULL : storer->GetObject() ); }

#endif

