/*****************************************************************************\
*                                                                             *
*  Name   : string_table                                                      *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2000-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "string_table.h"
#include "symbol_table.h"

#include <basis/function.h>
#include <basis/istring.h>
#include <basis/log_base.h>

istring string_table::text_form(bool show_nums) const
{
  istring output;
  for (int i = 0; i < symbols(); i++) {
    if (show_nums) output += isprintf("%5d: ", i);
    output += isprintf("%s=%s%s", name(i).s(), operator[](i).s(),
        log_base::platform_ending());
  }
  return output;
}

bool string_table::operator ==(const string_table &to_compare) const
{
  if (to_compare.symbols() != symbols()) return false;
  for (int i = 0; i < symbols(); i++) {
    const istring &key = name(i);
    istring *str1 = find(key);
    istring *str2 = to_compare.find(key);
    if (!str2) return false;
    if (*str1 != *str2) return false;
  }
  return true;
}

int string_table::packed_size() const
{
  int size = sizeof(int);
  for (int i = 0; i < symbols(); i++) {
    size += name(i).length();
    size += operator[](i).length();
  }
  return size;
}

void string_table::pack(byte_array &packed_form) const
{
  basis::attach(packed_form, symbols());
  for (int i = 0; i < symbols(); i++) {
    name(i).pack(packed_form);
    operator[](i).pack(packed_form);
  }
}

bool string_table::unpack(byte_array &packed_form)
{
  reset();
  int syms;
  if (!basis::detach(packed_form, syms)) return false;
  for (int i = 0; i < syms; i++) {
    istring name, content;
    if (!name.unpack(packed_form)) return false;
    if (!content.unpack(packed_form)) return false;
    outcome ret = add(name, content);
    if (ret != common::IS_NEW) return false;
  }
  return true;
}

