#ifndef STRING_TABLE_CLASS
#define STRING_TABLE_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : string_table                                                      *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2000-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "data_structure_dll.h"
#include "symbol_table.cpp"

#include <basis/istring.h>
#include <basis/packable.h>

//! Provides a symbol_table that holds strings as the content.
/*! This is essentially a table of named strings. */

class DATA_STRUCTURE_CLASS_STYLE string_table
: public symbol_table<istring>, public packable
{
public:
  string_table(int max_bits = 7) : symbol_table<istring>(max_bits) {}
    /*!< the "max_bits" specifies how wide the key space is for the strings
    (it works out to be 2^max_bits).  the default leads to a table that will
    easily accomodate 128 random strings efficiently. */
  virtual ~string_table() {}

  bool operator ==(const string_table &to_compare) const;
  bool operator !=(const string_table &to_compare) const
          { return !(*this == to_compare); }

  istring text_form(bool show_nums = true) const;
    //!< prints the contents of the table into the returned string.
    /*!< if "show_nums" is true, then indices will be printed in front of the
    lines. */

  virtual int packed_size() const;
  virtual void pack(byte_array &packed_form) const;
  virtual bool unpack(byte_array &packed_form);
};

#endif

