/*****************************************************************************\
*                                                                             *
*  Name   : table_configurator                                                *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2001-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "string_table.h"
#include "table_configurator.h"

#include <basis/common_templates.h>
#include <basis/istring.h>
#include <basis/utility.h>

class table_o_string_tables : public symbol_table<string_table>
{
public:
};

////////////////////////////////////////////////////////////////////////////

table_configurator::table_configurator(treatment_of_defaults behavior)
: configurator(behavior),
  _real_table(new table_o_string_tables)
{}

table_configurator::~table_configurator()
{
  WHACK(_real_table);
}

void table_configurator::reset() { _real_table->reset(); }

bool table_configurator::section_exists(const istring &section)
{ return !!_real_table->find(section); }

void table_configurator::sections(string_array &to_fill)
{
  to_fill.reset();
  for (int i = 0; i < _real_table->symbols(); i++)
    to_fill += _real_table->name(i);
}

bool table_configurator::delete_section(const istring &section)
{ return _real_table->whack(section) == common::OKAY; }

bool table_configurator::delete_entry(const istring &section,
    const istring &ent)
{
  string_table *sect = _real_table->find(section);
  if (!sect) return false;
  return sect->whack(ent) == common::OKAY;
}

bool table_configurator::put(const istring &section,
    const istring &entry, const istring &to_store)
{
  if (!to_store.length()) return delete_entry(section, entry);
  else if (!entry.length()) return delete_section(section);
  string_table *sect = _real_table->find(section);
  if (!sect) {
    // none exists yet, so add one.
    _real_table->add(section, string_table());
    sect = _real_table->find(section);
  }
  sect->add(entry, to_store);
  return true;
}

bool table_configurator::get(const istring &section,
    const istring &entry, istring &found)
{
  found = "";
  string_table *sect = _real_table->find(section);
  if (!sect) return false;
  istring *looked = sect->find(entry);
  if (!looked) return false;
  found = *looked;
  return true;
}

bool table_configurator::get_section(const istring &section,
    string_table &info)
{
  info.reset();
  string_table *sect = _real_table->find(section);
  if (!sect) return false;
  info = *sect;
  return true;
}

bool table_configurator::put_section(const istring &section,
    const string_table &info)
{
  string_table *sect = _real_table->find(section);
  if (!sect) {
    // none exists yet, so add one.
    _real_table->add(section, string_table());
    sect = _real_table->find(section);
  }
  *sect = info;
  return true;
}

