/*****************************************************************************\
*                                                                             *
*  Name   : reader_writer_lock                                                *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1998-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "mechanisms_implementation_only.h"
#include "rw_lock.h"

#include <basis/mutex.h>

//#define DEBUG_RW_LOCK
  // uncomment if you want the debugging version of the reader_writer_lock.

//hmmm: implement logging...
#undef LOG
#ifdef DEBUG_RW_LOCK
  #define LOG(to_print) CLASS_EMERGENCY_LOG(program_wide_logger(), to_print)
#else
  #define LOG(to_print) {}
#endif

reader_writer_lock::reader_writer_lock()
: _write_lock(new mutex()),
  _reader_count(0),
  _variable_lock(new mutex())
{
}

reader_writer_lock::~reader_writer_lock()
{
  delete _write_lock;
  _write_lock = NIL;
  delete _variable_lock;
  _variable_lock = NIL;
}

void reader_writer_lock::begin_write()
{
  #define func "begin_write"
  LOG(istring(istring::SPRINTF, "writer: wanting write lock with %d readers.",
      _reader_count));
  _write_lock->lock(); 
  LOG(istring(istring::SPRINTF, "writer: got write lock with %d readers.",
      _reader_count));
  #undef func
}

void reader_writer_lock::end_write()
{ 
  #define func "end_write"
  LOG("writer: undoing write lock");
  _write_lock->unlock();
  #undef func
}

void reader_writer_lock::begin_read()
{
  _variable_lock->lock();
  LOG(istring(istring::SPRINTF, "reader: got variable lock with %d "
      "readers.", _reader_count));
  _reader_count++;
  if (_reader_count == 1) {
    LOG("reader: wanting write lock");
    _write_lock->lock();
    LOG("reader: got write lock");
  }
  LOG(istring(istring::SPRINTF, "++readers=%d", _reader_count));
  _variable_lock->unlock();
}

void reader_writer_lock::end_read()
{
  _variable_lock->lock();
  LOG(istring(istring::SPRINTF, "reader: got variable lock with %d readers.",
      _reader_count));
  _reader_count--;
  LOG(istring(istring::SPRINTF, "--readers=%d", _reader_count));
  if (_reader_count == 0) {
    LOG("reader giving up write lock");
    _write_lock->unlock();
    LOG("after reader gave up write lock");
  }
  _variable_lock->unlock();
  LOG("after reader gave up variable lock");
}

