#ifndef READER_WRITER_LOCK_CLASS
#define READER_WRITER_LOCK_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : reader_writer_lock                                                *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1998-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "mechanisms_dll.h"

//! Controls multiple threads that may read or write a shared object.
/*!
  The object allows an unlimited number of simultaneous readers as long as
  there are no writers.  Only one writer can operate on the object at a time.
  When there are no readers remaining, the writer may operate.  When there are
  no writers remaining, the readers may operate.  Starvation is not considered;
  if there are always readers vying for access to the list, then writers may
  starve.  If a writer writes for an extended period of time, then readers may
  starve.  These situations must be dealt with in the code using the rw lock.
*/

class MECHANISMS_CLASS_STYLE reader_writer_lock
{
public:
  reader_writer_lock();
  ~reader_writer_lock();

  void begin_read();
    //!< allows this thread to start reading the shared object.
    /*!< this will be paused if any thread currently owns a write lock on the
     object.  any number of readers may begin reading if no writers are
     writing.  every call to begin_read() must be matched with a call to
     end_read(). */
  void end_read();
    //!< indicates that this thread is done reading the object now.

  void begin_write();
    //!< allows this thread to start writing the shared object.
    /*!< one writer may operate on the object at a time, but only if no readers
    are present.  every call to begin_write() must eventually be followed by
    a call to end_write(). */
  void end_write();

private:
  mutex *_write_lock;  //!< the lock that allows writing.
  int _reader_count;  //!< the number of readers currently operating.
  mutex *_variable_lock;  //!< protects the reader_count from race conditions.
};

#endif

