/*****************************************************************************\
*                                                                             *
*  Name   : timer                                                             *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1991-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "mechanisms_implementation_only.h"
#include "time_stamp.h"
#include "timer.h"

#include <basis/definitions.h>
#include <basis/guards.h>

timer::timer()
: _status(UNSTARTED),
  _start_time(new time_stamp()),
  _stop_time(new time_stamp()),
  _total_so_far(0)
{}

timer::timer(const timer &to_copy)
: _status(UNSTARTED),
  _start_time(new time_stamp()),
  _stop_time(new time_stamp()),
  _total_so_far(0)
{ *this = to_copy; }

timer::~timer()
{
  _status = UNSTARTED;
  WHACK(_start_time);
  WHACK(_stop_time);
}

timer &timer::operator =(const timer &to_copy)
{
  if (this == &to_copy) return *this;
  *_start_time = *to_copy._start_time;
  *_stop_time = *to_copy._stop_time;
  _status = to_copy._status;
  _total_so_far = to_copy._total_so_far;
  return *this;
}

void timer::reset() { _status = UNSTARTED; _total_so_far = 0; }

int timer::milliseconds() { return common_measure(); }

void timer::start()
{
  if (_status == RUNNING) return;
  *_start_time = time_stamp();
  _status = RUNNING;
}

int timer::compute_diff(const time_stamp &t1, const time_stamp &t2)
{ return int(t2.value() - t1.value()); }

void timer::halt()
{
  if (_status == STOPPED) return;
  else if (_status == UNSTARTED) return;

  *_stop_time = time_stamp();
  _total_so_far += compute_diff(*_start_time, *_stop_time);

  _status = STOPPED;
}

int timer::common_measure()
{
  bool restart = false;
  int to_return = 0;
  switch (_status) {
    case UNSTARTED: break;
    case RUNNING:
      // stop timer, restart afterwards.
      halt();
      restart = true;
      // intentional fall through to default.
    default:
      // set the return value to the accumulated time.
      to_return = _total_so_far;
      break;
  }
  if (restart) start();  // crank the timer back up if we were supposed to.
  return to_return;
}

