/*****************************************************************************\
*                                                                             *
*  Name   : file_time                                                         *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1992-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "filetime.h"

#include <basis/guards.h>
#include <basis/istring.h>

#undef LOG
#define LOG(a) {}
//temp until logging added.

file_time::file_time(FILE *the_file)
: am_i_valid(false)
{ reset(the_file); }

file_time::file_time(const istring &filename)
: am_i_valid(false)
{
  FILE *ptr = fopen(filename.s(), "r");
  reset(ptr);
  fclose(ptr);
}

file_time::~file_time() { am_i_valid = false; }

bool file_time::valid() const { return am_i_valid; }

void file_time::reset(FILE *the_file)
{
  if (!the_file) {
    am_i_valid = false;
    return;
  }
#if defined(__BCPLUSPLUS__)
  getftime(fileno(the_file), &t);
#elif defined(_MSC_VER)
  struct stat stat_buffer;
  if (fstat(fileno(the_file), &stat_buffer) < 0) {
    LOG(class_name() + istring("::get_time: stat failure on file"));
  }
  t = stat_buffer.st_mtime;
#elif defined(__UNIX__)
  struct stat stat_buffer;
  if (fstat(fileno(the_file), &stat_buffer) < 0) {
    LOG(class_name() + istring("::get_time: stat failure on file"));
  }
  t = stat_buffer.st_mtime;
#endif
  am_i_valid = true;
}

int file_time::compare(const file_time &b) const
{
#if defined(__BCPLUSPLUS__)
#ifdef DEBUG_FILE_TIME
  cout << "first " << t.ft_hour << ":" << t.ft_min << ":" << t.ft_tsec
    << " on " << t.ft_month << "/" << t.ft_day << " in "
      << 1980 + t.ft_year << endl << flush;
  cout << "second " << b.t.ft_hour << ":" << b.t.ft_min << ":" << b.t.ft_tsec
    << " on " << b.t.ft_month << "/" << b.t.ft_day << " in "
      << 1980 + b.t.ft_year << endl << flush;
#endif
  int a_date = t.ft_day + t.ft_month * 32 + t.ft_year * 32 * 13;
  int b_date = b.t.ft_day + b.t.ft_month * 32 + b.t.ft_year * 32 * 13;
  if (a_date > b_date) return 1; else if (a_date < b_date) return -1;

  int a_time = t.ft_min + t.ft_hour * 62;
  int b_time = b.t.ft_min + b.t.ft_hour * 62;
  if (a_time > b_time) return 1; else if (a_time < b_time) return -1;

  if (t.ft_tsec > b.t.ft_tsec) return 1;
  else if (t.ft_tsec == b.t.ft_tsec) return 0;
  else return -1;
#elif defined(__UNIX__) || defined(_MSC_VER)
  if (t > b.t) return 1;
  else if (t == b.t) return 0;
  else return -1;
#endif
}

bool file_time::operator < (const file_time &ft2) const
  { return bool(compare(ft2) < 0); }

bool file_time::operator > (const file_time &ft2) const
  { return bool(compare(ft2) > 0); }

bool file_time::operator <= (const file_time &ft2) const
  { return bool(compare(ft2) <= 0); }

bool file_time::operator >= (const file_time &ft2) const
  { return bool(compare(ft2) >= 0); }

bool file_time::operator != (const file_time &ft2) const
  { return bool(compare(ft2) != 0); }

bool file_time::operator == (const file_time &ft2) const
  { return bool(compare(ft2) == 0); }

