#ifndef HEAVY_FILE_OPERATIONS_CLASS
#define HEAVY_FILE_OPERATIONS_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : heavy file operations                                             *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Provides serious file operations, such as copy and partial writing.      *
*                                                                             *
*******************************************************************************
* Copyright (c) 2005-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "opsysdll.h"

#include <basis/object_base.h>

// forward.
class file_transfer_header;
class filename_list;

class OPSYSTEM_CLASS_STYLE heavy_file_operations
{
public:
  virtual ~heavy_file_operations() {}
  enum outcomes {
    OKAY = common::OKAY,
    BAD_INPUT = common::BAD_INPUT,
    DEFINE_OUTCOME(SOURCE_MISSING, -47, "The source file is not accessible"),
    DEFINE_OUTCOME(TARGET_DIR_ERROR, -48, "The target's directory could not "
        "be created"),
    DEFINE_OUTCOME(TARGET_ACCESS_ERROR, -49, "The target file could not be "
        "created"),
  };
  static const char *outcome_name(const outcome &to_name);

  IMPLEMENT_CLASS_NAME("heavy_file_operations");

  static bool recursive_create_dir(const istring &directory_name);
    // returns true if the "directory_name" could be created or already
    // existed.  false indicates that the operating system wouldn't let us
    // make the directory.

  static outcome copy_file(const istring &source, const istring &destination);
    // copies a file from the "source" location to the "destination".
    // the outcomes could be from this class or from common::outcomes.

  static outcome write_file_chunk(const istring &target, int byte_start,
          const byte_array &chunk, bool truncate = true);
    // writes the portion of the file "target" that's stored in "chunk" into
    // the file at position "byte_start".  the entire "chunk" will be used,
    // which means the file will either be that much larger or have the space
    // between byte_start and (byte_start + chunk.length() - 1) replaced.
    // if the file is not yet as large as "byte_start", then it will be
    // expanded appropriately.  if "truncate" is true, then any contents past
    // the new chunk are dropped from the file.

  static outcome buffer_files(const istring &source_root,
          const filename_list &to_transfer, file_transfer_header &last_action,
          byte_array &storage, int maximum_bytes);
    // reads the files in "to_transfer" and packs them into a "storage" buffer.
    // the maximum size allowed in storage is "maximum_bytes".  the record
    // of the last file piece stored in "last_action" allows the next chunk
    // to be sent in subsequent calls.  note that the buffer "storage" is
    // cleared by this call; this is not an additive operation.
};

////////////////////////////////////////////////////////////////////////////

// this describes an ongoing file transfer.

class OPSYSTEM_CLASS_STYLE file_transfer_header : public packable
{
public:
  istring _filename;  // the name of the file being transferred.
  int _byte_start;  // the starting location in the file being sent.
  int _length;  // the length of the transferred piece.

  file_transfer_header();

  virtual void pack(byte_array &packed_form) const;
  virtual bool unpack(byte_array &packed_form);

  virtual int packed_size() const;

  istring text_form() const;
};

#endif

