#ifndef NULL_LOGGER_CLASS
#define NULL_LOGGER_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : null_logger                                                       *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Provides a log_base object that throws away all of the strings provided  *
*  to it.  This is useful when an interface requires a log_base object but    *
*  one does not wish to generate an output file.  This object is equivalent   *
*  to /dev/null in Unix and nul: in Win32.                                    *
*                                                                             *
*******************************************************************************
* Copyright (c) 2001-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "opsysdll.h"

#include <basis/log_base.h>

class OPSYSTEM_CLASS_STYLE null_logger : public log_base
{
public:
  null_logger() {}
  virtual ~null_logger() {}

  IMPLEMENT_CLASS_NAME("null_logger");

  virtual outcome log(const istring &info, int filter = ALWAYS_PRINT)
      { return common::OKAY; }

private:
};

////////////////////////////////////////////////////////////////////////////

// retasks the program wide logger to use a new null_logger.  the original
// log_base object is returned and must be dealt with appropriately.
log_base OPSYSTEM_FUNCTION_STYLE *set_PW_logger_for_null();

// a macro combining all the defaults to retask the program-wide logger as
// a null_logger.
#ifndef OMIT_PROGRAM_WIDE_LOGGER
  #define SET_DEFAULT_NULL_LOGGER { \
    log_base *old_log = set_PW_logger_for_null(); \
    WHACK(old_log); \
  }
#else
  #define SET_DEFAULT_NULL_LOGGER
#endif

#endif

