#ifndef PATH_CONFIGURATION_GROUP
#define PATH_CONFIGURATION_GROUP

/*****************************************************************************\
*                                                                             *
*  Name   : path configuration definitions                                    *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    These define installation-wide locations in the file system.             *
*                                                                             *
*******************************************************************************
* Copyright (c) 2000-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "opsysdll.h"

#define OSFUNC OPSYSTEM_FUNCTION_STYLE

namespace path_configuration {

const char OSFUNC *PATH_CONFIGURATION_FILENAME();
  // the configuration file stores paths that the low-level libraries need for
  // proper operation.  this is just the base filename; the fully qualified
  // path to the path configuration file is provided below.

istring OSFUNC path_configuration_file(bool consult_environment = true);
  // this is the fully specified path to the config file, as determined by
  // the directory where this executable is running.  if the "consult_
  // environment" flag is true, then the two environment variables HOOPLE_APPS
  // and INOVA_APPS will be used for the exe directory if the config file
  // cannot be found in the executable's own directory.

istring OSFUNC application_name();
  // returns the name of "this" program.

void OSFUNC hard_code_application_name(const istring &new_name);
  // allows specification of the application name in cases where it is not
  // correct or appropriate.  after this is called, the application_name()
  // method will return the "new_name".

istring OSFUNC installation_root(bool consult_environment = true);
  //!< returns the top-level directory of this installation.
  /*!< returns the folder containing the application directory (usually) as
  well as the other folders of configuration information and fonts and
  such needed by this installation.  if "consult_environment" is true,
  then the environment variables are searched for "HOOPLE_APPS" and
  "INOVA_APPS" to find the program directory where the main paths ini
  file is located. */

istring OSFUNC application_directory(bool consult_environment = true);
  // returns the name of the directory where this application is running from.
  // if "consult_environment" is true, then the environment is searched for
  // "HOOPLE_APPS" and "INOVA_APPS" to report the program directory.

void OSFUNC hard_code_application_dir(const istring &new_dir);
  // forces the application_directory() method to always return the "new_dir".
  // this can be changed during runtime but that would lead to some odd
  // results; it is recommended that this facility never be used except by
  // programs which do not live with the rest of the installed applications.

istring OSFUNC get_logging_directory();
  // returns the directory where log files will be stored.

istring OSFUNC make_logfile_name(const istring &base_name);
  // creates and returns a full path name for a log file with the "base_name"
  // specified, using the LOGGING_FOLDER_NAME entry as the directory name.  if
  // the logging directory does not exist, it is created if it can be.  the
  // returned name is suitable for logging mechanisms that need a filename.
  // an empty string is returned on failure to create the directory.

istring OSFUNC read_item(const istring &key_name);
  // returns the entry listed under the "key_name".  if the "key_name" is
  // valid for the root configuration file, then this should always return
  // an appropriate value.  a failure is denoted by return of an empty string.

// the following are all key names within the path configuration file.

const istring OSFUNC &GLOBAL_SECTION_NAME();
  // within the configuration file, there is a section named as above.
  // this section should only be used to define path configuration entries that
  // affect the general operation of the system.  entries that are specific
  // to particular programs or subsystems should be contained in their own
  // section.

const istring OSFUNC &LOCAL_FOLDER_NAME();
  // provides the entry name in the config file that points at the root
  // of the installation.  this is where all files for this product are
  // stored on "this" machine.

const istring OSFUNC &LOGGING_FOLDER_NAME();
  // this is where the log files for the system are to be stored.  this is
  // always considered to be a directory under the local folder.  the
  // make_logfile_name() function (see below) can be used to create a properly
  // formed filename for logging.

} // namespace.

#undef OSFUNC

#endif

