#ifndef HOOPLE_STARTUP_GROUP
#define HOOPLE_STARTUP_GROUP

/*****************************************************************************\
*                                                                             *
*  Name   : HOOPLE startup support                                            *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    The HOOPLE_STARTUP_CODE macro must be deployed once and only once in     *
*  each program that uses the HOOPLE libraries.  The macro ensures that some  *
*  critical objects are defined and created properly for the program's use.   *
*                                                                             *
*******************************************************************************
* Copyright (c) 1994-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include <basis/portable.h>
#include <basis/utility.h>

// forward.
class static_memory_gremlin;

namespace startup_code {

bool PWL_EXPO program_is_dying();
  //!< true if the program is trying to exit.

const char PWL_EXPO *EXCEPTION_INFO_FOR_SHUTTING_DOWN();
  // this string will be thrown as an exception if a program is trying to
  // use global objects when it should be shutting down instead.

class PWL_EXPO library_wide_cleanup
{
public:
  library_wide_cleanup();
    //!< must be called to properly set up the library support.

  ~library_wide_cleanup();

private:
  static_memory_gremlin *_shared_chunks;  //!< hooks to the static objects.
};

class PWL_EXPO program_wide_cleanup
{
public:
  program_wide_cleanup();
  ~program_wide_cleanup();
};

// HOOPLE_STARTUP_CODE_DLL: must be deployed in root file of dynamic libraries.
// (the file <basis/dll_root.cpp> can be used for most of the dll setup.)
#define HOOPLE_STARTUP_CODE_DLL \
  DEFINE_ARGC_AND_ARGV; /* any OS specific code. */ \
  DEFINE_INSTANCE_HANDLE;  /* set up the instance handle. */ \
  static startup_code::library_wide_cleanup _jean_reno

// HOOPLE_STARTUP_CODE: must be deployed in the root application's file
// (where main, WinMain, CWinApp, etc. reside) to ensure that the
// static synchronization scheme is properly initialized.
#define HOOPLE_STARTUP_CODE \
  HOOPLE_STARTUP_CODE_DLL; \
  /* the creation of this static cleanup object also establishes the \
   * static objects that are provided as global hoople services. */ \
  static startup_code::program_wide_cleanup _harvey_keitel

// legacy macros from the time that all of the startup code lived in the
// basis library.
#define BASIS_STARTUP_CODE_DLL HOOPLE_STARTUP_CODE_DLL
#define BASIS_STARTUP_CODE HOOPLE_STARTUP_CODE

} // namespace.

#endif

