#ifndef PERIODICITY_CLASS
#define PERIODICITY_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : periodicity                                                       *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Describes the scheduling characteristics of an item to be scheduled,     *
*  in terms of time of entry, repetition interval and expiration.             *
*                                                                             *
*******************************************************************************
* Copyright (c) 1997-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "scheduling_dll.h"

#include <mechanisms/time_stamp.h>

// all "int" measurements here are in milliseconds.

class SCHEDULING_CLASS_STYLE periodicity
{
public:
  int entry_interval;
    // the time interval between entries into the scheduler.

  time_stamp start_time;
    // the time at which this item can be scheduled; it will not be activated
    // before then.  this is reset after every time the item is scheduled such
    // that it always indicates the correct next starting time (by adding the
    // interval amount to the time at which the last schedule activation
    // occurred).

  int expiration_interval;
    // if this is non-zero, then it is the interval between expirations of the
    // scheduled item.  when the item expires, its expiration_hook is called.
    // the hook decides either that the item will remain in the schedule until
    // the next expiration or that it gets chucked out of the schedule.
    // whenever a scheduling activation happens successfully on the item,
    // the expiration timer is reset to this interval.  thus it is a
    // per-activation expiration
//and hence is useless, since each activation is seemingly instantaneous
//to the scheduler.  what the heck!
// how is the expiration interval computed??
// is it the start time plus the interval?  meaning that whenever an item
// gets scheduled, the expiration countdown timer is reset to zero?
// the expiration timer should be reset whenever the item enters the schedule,
// because right now scheduling is pretty much allowed to continue as long
// as the item is behaving nicely...  is that okay???  seems like the
// expiration thing is a sham.
// but then again, it will make us consider timing issues.  is 42 seconds
// really long enough for any _full_ transmission?  that's what the current
// limit on packets being sent is, and isn't that the same as the overall time
// here?

  periodicity(int entry = 0, const time_stamp &start = time_stamp(),
          int expire = 0)
  : entry_interval(entry), start_time(start), expiration_interval(expire) {}
    // creates a period with the "entry" interval, the specified "start" time,
    // and "expire" milliseconds allowed in the schedule before expiration
    // occurs.

  istring per_text_form() const;
    // returns a string containing a printout of the class data.
};

#endif

