#ifndef SCHEDULING_COMMON_CLASS
#define SCHEDULING_COMMON_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : scheduling common definitions                                     *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Defines values used within the scheduling library.                       *
*                                                                             *
*******************************************************************************
* Copyright (c) 1991-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "scheduling_dll.h"

#include <basis/object_base.h>

// forward.
class schedulable;

class SCHEDULING_CLASS_STYLE scheduling_common : public virtual object_base
{
public:
  // these outcomes represent what can happen during an attempt to schedule
  // an item or during other processes on the item...
  //   OKAY:      the item was scheduled just fine.
  //   POSTPONE:  the item should be skipped for now.
  //   CANCEL:    the item should be removed from the schedule.
  //   FINISHED:  the item is done being scheduled.
  //   TIME_OUT:  the item has timed out during scheduling.
  //   FAILURE:   there was a failure during the scheduling of the item.
  enum outcomes {
    OKAY = common::OKAY,
    UNINITIALIZED = common::UNINITIALIZED,
    TIME_OUT = common::TIMED_OUT,
    FAILURE = common::FAILURE,
    NOT_FOUND = common::NOT_FOUND,
    BAD_INPUT = common::BAD_INPUT,
    NO_SPACE = common::NO_SPACE,

    DEFINE_OUTCOME(POSTPONE, -61, "The operation is postponed; try again "
        "later"),
    DEFINE_OUTCOME(CANCEL, -62, "This item has been cancelled"),
    DEFINE_OUTCOME(FINISHED, -63, "This item's scheduling is complete"),
    DEFINE_OUTCOME(MORE_LEFT, -64, "There are still more items left to "
        "schedule that didn't fit in current timeslice"),
    DEFINE_OUTCOME(NO_ACTOR, -65, "The actor responsible for this item was "
        "missing."),
  };

  IMPLEMENT_CLASS_NAME("scheduling_common");

  static const char *outcome_name(const outcome &to_name);
    // returns the textual form of the outcome "to_name".
};

#endif

