#ifndef FILE_TRANSFER_INFOTON_CLASS
#define FILE_TRANSFER_INFOTON_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : file_transfer_infoton                                             *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    These are the base objects used by the file transfer tentacle to         *
*  schedule file transfers and transfer file pieces.                          *
*                                                                             *
*  Note: this is a fairly heavy-weight header.                                *
*                                                                             *
*******************************************************************************
* Copyright (c) 2005-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "dll_tentacles.h"

#include <basis/byte_array.h>
#include <octopus/infoton.h>

// forward.
class directory_tree;

class TENTACLES_CLASS_STYLE file_transfer_infoton : public infoton
{
public:
  enum commands {
    TREE_COMPARISON = 1,
      // the destination root will be compared with the source root.  the
      // packed data in the request holds a packed symbol tree describing the
      // destination hierarchy.  the packed data in the response is the packed
      // filename list that represents the differences between the two
      // hierarchies.  this is considered to start a file transfer based on
      // those differences.
    PLACE_FILE_CHUNKS,
      // the destination side requests a new set of chunks, based on the
      // source's memory of where the transfer is at.  this will only perform
      // properly when the file transfer was requested to be started by the
      // client using a TREE_COMPARISON request.  the request has an empty data
      // chunk, but the response consists of an arbitrary number of pairs of
      // [ file_transfer_header + file chunk described in header ].
  };

  outcome _success;  // reports what kind of result occurred.
  bool _request;  // if it's not a request, then it's a response.
  byte _command;  // one of the commands above.
  istring _src_root;  // the top-level directory of the source.
  istring _dest_root;  // the top-level directory of the destination.
  byte_array _packed_data;  // the packed headers and file chunks.

  file_transfer_infoton();

  file_transfer_infoton(const outcome &success, bool request, commands command,
          const istring &source, const istring &destination,
          const byte_array &packed_data);

  virtual ~file_transfer_infoton();

  virtual void pack(byte_array &packed_form) const;
  virtual bool unpack(byte_array &packed_form);

  void package_tree_info(const directory_tree &tree,
          const string_array &includes);
    // prepares the packed data from the "tree" and "includes" list.

  virtual clonable *clone() const
          { return cloner<file_transfer_infoton>(*this); }

  virtual int packed_size() const;

  static const string_array &file_transfer_classifier();
    // returns the classifier for this type of infoton.
};

#endif

