/*****************************************************************************\
*                                                                             *
*  Name   : column_headers                                                    *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2000-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "column_headers.h"

#include <basis/array.h>
#include <basis/common_templates.h>
#include <basis/istring.h>
#include <basis/utility.h>

struct column_info {
  istring _name;
  column_headers::justifications _just;
  int _max_width;

  column_info(const istring &name = "",
      column_headers::justifications just = column_headers::LEFT_JUSTIFY,
      int max_width = -1) : _name(name), _just(just), _max_width(max_width) {}
};

class column_info_array : public array<column_info> {};

column_headers::column_headers()
: _cols(new column_info_array)
{
}

column_headers::~column_headers()
{
  WHACK(_cols);
}

int column_headers::columns() const { return _cols->length(); }

///  enum justifications { LEFT_JUSTIFY, CENTER_JUSTIFY, RIGHT_JUSTIFY };

void column_headers::add(const istring &column_name,
    justifications placement, int maximum_width)
{
}

bool column_headers::get(int index, istring &column_name,
    justifications &placement, int &maximum_width)
{
return false;
}

bool column_headers::zap(int start, int end)
{
  if (negative(start)) return false;
  if (start >= columns()) return false;
  if (negative(end)) { _cols->zap(start, start); return true; }
  if (end >= columns()) return false;
  _cols->zap(start, end);
  return true;
}

bool column_headers::format(istring &to_fill, const string_array &to_format)
{
return false;
}

bool column_headers::format(istring &to_fill, const string_matrix &to_format)
{
return false;
}

void column_headers::pack(byte_array &packed_form) const
{
  basis::attach(packed_form, columns());
  for (int i = 0; i < columns(); i++) {
    column_info &info = _cols->operator [] (i);
    basis::attach(packed_form, int(info._just));
    basis::attach(packed_form, info._max_width);
    info._name.pack(packed_form);
  }
}

bool column_headers::unpack(byte_array &packed_form)
{
  int cols;
  if (!basis::detach(packed_form, cols)) return false;
  // make columns large enough for packed list.
  _cols->reset(cols);
  for (int i = 0; i < cols; i++) {
    column_info &info = _cols->operator [] (i);
    int just;
    if (!basis::detach(packed_form, just)) return false;
    info._just = justifications(just);
    if (!basis::detach(packed_form, info._max_width)) return false;
    if (!info._name.unpack(packed_form)) return false;
  }
  return true;
}

