#ifndef FILE_DIFFER_H
#define FILE_DIFFER_h

/*****************************************************************************\
*
*  Name   : file_differ
*  Author : Aaron Buchanan
*
*******************************************************************************
* Copyright (c) 2002-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

//! Provide utilities for comparing two files and seeing their differences.
/*!
  The file_differ executes a program named "diff".  A program such as
  "diff.exe" or "diff.bat" be present in your path for the file_differ
  to work.  This has been tested and used with the diff program
  installed as part of the cygwin package.
*/

#include "textual_dll.h"

namespace file_differ
{

    bool TEXTUAL_FUNCTION_STYLE diff_files(const istring &_file1, const istring &_file2, const istring &_options,
                    istring &_results, istring &_error);
        //!< uses the "diff" program to compare _file1 and _file2.
        /*!< the results of the comparison are stored in _results.  parameters to the diff program
        can be provided in _options.
        if an error occurs false is returned and the _error string will contain
        a description of the error. */

    void TEXTUAL_FUNCTION_STYLE AddCRs(istring &_string);
        //!< adds carriage returns "\r" to any newlines without them in the string
}

#endif

